@@ -31,8 +31,8 @@ KEY_LENGTH = 16
 
 
 # Starting entrypoint index
-# 0 is set_key, 1 is attest, 2 is handle_input, 3 is handle_handler
-START_ENTRY_INDEX = 4
+# 0 is set_key, 1 is attest, 2 is exit, 3 is handle_input, 4 is handle_handler
+START_ENTRY_INDEX = 5
 # Starting indexes of inputs, outputs, requests and handlers
 # They need to have different indexes, because the `index` field in Connection does
 # not distinguish between them. If the same index is used for different types, bad
@@ -52,21 +52,21 @@ START_HANDLER_INDEX = 49152
 REGEX_LAZY = "^\s*#\s*[\s*macro_use\s*]\s*extern\s*crate\s*lazy_static\s*;"
 
 REGEX_OUTPUT = ("^[ \t]*//@[ \t]*sm_output[ \t]*\([ \t]*"
-                "(?P<fname>_?[a-zA-Z]+[_a-zA-Z0-9]*)[ \t]*\)[ \t]*$")
+                "(?P<fname>[_a-zA-Z]+[_a-zA-Z0-9]*)[ \t]*\)[ \t]*$")
 
 REGEX_INPUT = ("^[ \t]*//@[ \t]*sm_input[ \t]*\n\s*pub\s+fn\s+"
-               "(?P<fname>_?[a-zA-Z]+[_a-zA-Z0-9]*)\s*\(\s*_?"
-               "[a-zA-Z]+[_a-zA-Z0-9]*\s*:\s*&\s*\[\s*u8\s*]\s*\)\s*\{")
+               "(?P<fname>[_a-zA-Z]+[_a-zA-Z0-9]*)\s*\(\s*"
+               "[_a-zA-Z]+[_a-zA-Z0-9]*\s*:\s*&\s*\[\s*u8\s*]\s*\)\s*\{")
 
 REGEX_ENTRY = ("^[ \t]*//@[ \t]*sm_entry[ \t]*\n\s*pub\s+fn\s+"
-               "(?P<fname>_?[a-zA-Z]+[_a-zA-Z0-9]*)\s*\(\s*_?"
-               "[a-zA-Z]+[_a-zA-Z0-9]*\s*:\s*&\s*\[\s*u8\s*]\s*\)\s*->\s*"
+               "(?P<fname>[_a-zA-Z]+[_a-zA-Z0-9]*)\s*\(\s*"
+               "[_a-zA-Z]+[_a-zA-Z0-9]*\s*:\s*&\s*\[\s*u8\s*]\s*\)\s*->\s*"
                "ResultMessage\s*\{")
 
-REGEX_REQUEST = ("^[ \t]*//@[ \t]*sm_request[ \t]*\([ \t]*(?P<fname>_?[a-zA-Z]+"
+REGEX_REQUEST = ("^[ \t]*//@[ \t]*sm_request[ \t]*\([ \t]*(?P<fname>[_a-zA-Z]+"
                  "[_a-zA-Z0-9]*)[ \t]*\)[ \t]*$")
 
 REGEX_HANDLER = ("^[ \t]*//@[ \t]*sm_handler[ \t]*\n\s*pub\s+fn\s+"
-                 "(?P<fname>_?[a-zA-Z]+[_a-zA-Z0-9]*)\s*\(\s*_?[a-zA-Z]+"
+                 "(?P<fname>[_a-zA-Z]+[_a-zA-Z0-9]*)\s*\(\s*[_a-zA-Z]+"
                  "[_a-zA-Z0-9]*\s*:\s*&\s*\[\s*u8\s*]\s*\)\s*->\s*"
                  "Vec\s*<\s*u8\s*>\s*\{")
