@@ -10,7 +10,6 @@ pub mod authentic_execution {
     use reactive_net::{ResultCode, CommandCode, ResultMessage, CommandMessage, EntrypointID};
     use reactive_crypto::Encryption;
     use crate::__run::MODULE_KEY;
-    #[cfg(feature = "measure_time")]
     use std::time::{SystemTime, UNIX_EPOCH};
 
     #[derive(Debug)]
@@ -144,16 +143,29 @@ pub mod authentic_execution {
         }};
     }
 
-    #[cfg(feature = "measure_time")]
-    pub fn measure_time(msg : &str) {
+    #[allow(dead_code)]
+    pub fn measure_time_ms(msg : &str) {
+        match SystemTime::now().duration_since(UNIX_EPOCH) {
+            Ok(d)   => info!(&format!("{}: {} ms", msg, d.as_millis())),
+            Err(_)  => info!(&format!("{}: ERROR", msg))
+        }
+    }
+
+    #[allow(dead_code)]
+    pub fn measure_time_us(msg : &str) {
         match SystemTime::now().duration_since(UNIX_EPOCH) {
             Ok(d)   => info!(&format!("{}: {} us", msg, d.as_micros())),
             Err(_)  => info!(&format!("{}: ERROR", msg))
         }
     }
 
+    #[cfg(feature = "measure_time")]
+    fn _measure_time(msg : &str) {
+        measure_time_us(msg);
+    }
+
     #[cfg(not(feature = "measure_time"))]
-    pub fn measure_time(_msg : &str) {}
+    fn _measure_time(_msg : &str) {}
 
     /// This is the only interface to the software module from outside
     /// Each request has to be sent to this function
@@ -189,6 +201,11 @@ pub mod authentic_execution {
     fn set_key(enc : u8, conn_id : &[u8], index : &[u8], nonce : &[u8], cipher : &[u8]) -> ResultMessage {
         // The tag is included in the cipher
 
+        let nonce_u16 = data_to_u16(nonce);
+        if !check_nonce(nonce_u16) {
+            return failure(ResultCode::IllegalPayload, None)
+        }
+
         let mut ad = vec!(enc);
         ad.extend_from_slice(conn_id);
         ad.extend_from_slice(index);
@@ -205,6 +222,8 @@ pub mod authentic_execution {
            Err(_)   => return failure(ResultCode::CryptoError, None)
         };
 
+        increment_nonce();
+
         let enc_type = match Encryption::from_u8(enc) {
             Some(e) => e,
             None    => return failure(ResultCode::CryptoError, None)
@@ -258,7 +277,7 @@ pub mod authentic_execution {
             None => return failure(ResultCode::BadRequest, None)
         };
 
-        measure_time("handle_input_before_decryption");
+        _measure_time("handle_input_before_decryption");
 
         let nonce = conn.get_nonce();
         let data = match reactive_crypto::decrypt(payload, &conn.get_key(), &u16_to_data(nonce), &conn.get_encryption()) {
@@ -270,7 +289,7 @@ pub mod authentic_execution {
         let index = &conn.get_index();
         drop(map); // release map as soon as we don't need it anymore
 
-        measure_time("handle_input_after_decryption");
+        _measure_time("handle_input_after_decryption");
 
         let handler = match INPUTS.get(index) {
             Some(h) => h,
@@ -279,7 +298,7 @@ pub mod authentic_execution {
 
         handler(&data);
 
-        measure_time("handle_input_after_handler");
+        _measure_time("handle_input_after_handler");
 
         success(None)
     }
@@ -305,7 +324,7 @@ pub mod authentic_execution {
             None => return failure(ResultCode::BadRequest, None)
         };
 
-        measure_time("handle_handler_before_1st_decryption");
+        _measure_time("handle_handler_before_1st_decryption");
 
         let nonce = conn.get_nonce();
         let key = conn.get_key();
@@ -325,7 +344,7 @@ pub mod authentic_execution {
         // release lock of map, so that it can be used by other threads
         drop(map);
 
-        measure_time("handle_handler_after_1st_decryption");
+        _measure_time("handle_handler_after_1st_decryption");
 
         // execute handler
         let handler = match HANDLERS.get(&index) {
@@ -335,7 +354,7 @@ pub mod authentic_execution {
 
         let result = handler(&data);
 
-        measure_time("handle_handler_after_handler");
+        _measure_time("handle_handler_after_handler");
 
         // encrypt response
         let response = match reactive_crypto::encrypt(&result, &key,
@@ -344,40 +363,45 @@ pub mod authentic_execution {
            Err(_)   => return failure(ResultCode::CryptoError, None)
         };
 
-        measure_time("handle_handler_after_2nd_encryption");
+        _measure_time("handle_handler_after_2nd_encryption");
 
         success(Some(response))
     }
 
-    pub fn exit_wrapper(data : &[u8]) -> ResultMessage  {
+    pub fn disable_wrapper(data : &[u8]) -> ResultMessage  {
         // The payload is: [nonce - cipher]
-        debug!("ENTRYPOINT: exit");
+        debug!("ENTRYPOINT: disable");
 
         if data.len() < 2 {
             return failure(ResultCode::IllegalPayload, None)
         }
 
-        exit(&data[0..2], &data[2..])
+        disable(&data[0..2], &data[2..])
     }
 
-    fn exit(nonce : &[u8], cipher : &[u8]) -> ResultMessage {
+    fn disable(nonce : &[u8], cipher : &[u8]) -> ResultMessage {
         // The tag is included in the cipher
-        
-        //TODO do not trust this nonce but keep an internal one
-        let mut ad = vec!();
-        ad.extend_from_slice(nonce);
+
+        let nonce_u16 = data_to_u16(nonce);
+        if !check_nonce(nonce_u16) {
+            return failure(ResultCode::IllegalPayload, None)
+        }
 
         let decoded_key = match base64::decode(&*MODULE_KEY) {
             Ok(k)   => k,
             Err(_)  => return failure(ResultCode::InternalError, None)
         };
 
-        if let Err(_) = reactive_crypto::decrypt(cipher, &decoded_key, &ad, &Encryption::Aes) {
+        if let Err(_) = reactive_crypto::decrypt(cipher, &decoded_key, nonce, &Encryption::Aes) {
             return failure(ResultCode::CryptoError, None)
         };
 
-        // exit
-        std::process::exit(0);
+        increment_nonce();
+
+        // delete all connections, making the module disabled in practice
+        delete_all_connections();
+
+        success(None)
     }
 
     #[allow(dead_code)] // this is needed if we have no outputs to avoid warnings
@@ -398,7 +422,7 @@ pub mod authentic_execution {
                 }
             };
 
-            measure_time("handle_output_before_encryption");
+            _measure_time("handle_output_before_encryption");
 
             let nonce = conn.get_nonce();
             let payload = match reactive_crypto::encrypt(data, &conn.get_key(),
@@ -410,7 +434,7 @@ pub mod authentic_execution {
                }
             };
 
-            measure_time("handle_output_after_encryption");
+            _measure_time("handle_output_after_encryption");
 
             conn.increment_nonce();
             let func = || drop(map);
@@ -418,7 +442,7 @@ pub mod authentic_execution {
                 error!(&format!("{}", e));
             }
 
-            measure_time("handle_output_after_dispatch");
+            _measure_time("handle_output_after_dispatch");
         }
     }
 
@@ -437,7 +461,7 @@ pub mod authentic_execution {
             None        => return Err(Error::InternalError) // it shouldn't happen
         };
 
-        measure_time("handle_request_before_1st_encryption");
+        _measure_time("handle_request_before_1st_encryption");
 
         // encrypt payload
         let nonce = conn.get_nonce();
@@ -456,7 +480,7 @@ pub mod authentic_execution {
         conn.increment_nonce();
         conn.increment_nonce();
 
-        measure_time("handle_request_after_1st_encryption");
+        _measure_time("handle_request_after_1st_encryption");
 
         // send payload:
         // drop map only after the message is sent to the EM.
@@ -468,7 +492,7 @@ pub mod authentic_execution {
             None        => return Err(Error::InternalError) //it should never happen
         };
 
-        measure_time("handle_request_after_response_received");
+        _measure_time("handle_request_after_response_received");
 
         // Check response
         let resp_body = match response.get_code() {
@@ -488,7 +512,7 @@ pub mod authentic_execution {
            Err(_)   => return Err(Error::CryptoError)
         };
 
-        measure_time("handle_request_after_2nd_decryption");
+        _measure_time("handle_request_after_2nd_decryption");
 
         Ok(data)
     }
@@ -549,6 +573,9 @@ pub mod authentic_execution {
         static ref REQUESTS: Mutex<HashMap<u16, u16>> = {
             Mutex::new(HashMap::new())
         };
+        static ref NONCE: Mutex<u16> = {
+            Mutex::new(0)
+        };
     }
 
     // Constants: Module's key, ID, Inputs, Outputs
@@ -558,6 +585,12 @@ pub mod authentic_execution {
         CONNECTIONS.lock().unwrap().insert(conn_id, conn);
     }
 
+    fn delete_all_connections() {
+        CONNECTIONS.lock().unwrap().clear();
+        OUTPUTS.lock().unwrap().clear();
+        REQUESTS.lock().unwrap().clear();
+    }
+
     fn add_output(out_id : u16, conn_id : u16) {
         let mut map = OUTPUTS.lock().unwrap();
 
@@ -590,4 +623,13 @@ pub mod authentic_execution {
             None        => None
         }
     }
+
+    fn increment_nonce() {
+        let mut nonce_ref = NONCE.lock().unwrap();
+        *nonce_ref += 1;
+    }
+
+    fn check_nonce(nonce : u16) -> bool {
+        *NONCE.lock().unwrap() == nonce
+    }
 }
